/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "config.h"

#include "global.h"
#include "game.h"
#include "load.h"
#include "client.h"


/* create home directory (~/.gtkatlantic) *
 * return TRUE if exist/created, return FALSE if cannot be created */
gboolean create_home_directory()  {

	guchar *path;
	struct stat s;

	path = g_strconcat( getenv("HOME"), "/", ".", PACKAGE, "/", NULL);
	global->path_home = g_strdup(path);

	if(stat(path, &s) < 0)  {

		mkdir(path, 0777);
		if(stat(path, &s) < 0)  {

			g_free(path);
			return FALSE;
		}
	}

	g_free(path);
	return TRUE;
}


/* read config files */
void read_config_files()   {

	guchar *filename;
	struct stat s;

	/* read default config file */
	filename = g_strconcat(PACKAGE_DATA_DIR, "/config.xml", NULL);
	if(! xmlparse_file_config(filename) ) exit(-1);
	g_free(filename);

	/* read user config file */
	filename = g_strconcat(global->path_home, "config.xml", NULL);
	if(stat(filename, &s) >= 0)
		xmlparse_file_config(filename);

	g_free(filename);
}


/* return a valid slot for a connection     *
 *   else return FALSE                      */
gboolean connect_get_valid_id(guint16 *idconnect)  {

	guint16 i;

	for(i = 1; i < MAX_CONNECTION; i++)
		if(!connection_is_open[i]) {

			*idconnect = i;
			return(TRUE);
		}

	return(FALSE);
}


/* this function made a new connection for metaserver (list & games) */
void create_connection_metaserver(guint32 gametype)  {

	guint16 connectid;
	gchar *sendstr;

	if(! client_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
	connection[connectid]->type = gametype;

	/* send client version (to retrieve info about release) */
	if(config->metaserver_sendclientversion)  {

		sendstr = g_strconcat("CLIENT", PACKAGE, VERSION, "\n", NULL);
		client_send(connectid, sendstr, strlen(sendstr));
		g_free(sendstr);
	}

	sendstr = g_strdup("LIST\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* this function made a new connection for get games only */
void create_connection_get_games(guchar *host, guint32 port)  {

	guint16 connectid;
	gchar *sendstr;

	if(! client_connect(&connectid, host, port) ) return;
	connection[connectid]->type = CONNECT_TYPE_MONOPD_GETGAME;

	sendstr = g_strdup(".gl\n");
	client_send(connectid, sendstr, strlen(sendstr));
	g_free(sendstr);
}


/* insert test in Message Box */ 
void text_insert_message(gchar* text, guint32 lenght)  {

	guint32 i, len;

	if(global->phase < PHASE_GETGAMES) return;


	gtk_text_freeze( GTK_TEXT(global->Message) );

	len = gtk_text_get_length( GTK_TEXT(global->Message) );
	gtk_text_set_point( GTK_TEXT(global->Message), len);
	if(len > 0)  gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, "\n", 1);
	gtk_text_insert(GTK_TEXT(global->Message), NULL, NULL, NULL, text, lenght);
	global->message_nb_lines++;

	while(global->message_nb_lines > config->message_max_lines) {

		/* if unlimited */
		if(config->message_max_lines <= 0)  break;

		len = gtk_text_get_length( GTK_TEXT(global->Message) );

		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(global->Message), i) != '\n' ; i++)  {

			if(i >= len)  { /* argl... no CR founded */

				gtk_text_set_point( GTK_TEXT(global->Message), 0);
				gtk_text_forward_delete( GTK_TEXT(global->Message), len);
				global->message_nb_lines = 0;
				break;
			}
		}

		if(global->message_nb_lines)  {

			i++;
			gtk_text_set_point( GTK_TEXT(global->Message), 0);
			gtk_text_forward_delete( GTK_TEXT(global->Message), i);
			global->message_nb_lines--;
		}
	}

	len = gtk_text_get_length( GTK_TEXT(global->Message) );
	gtk_text_set_point( GTK_TEXT(global->Message), len);

	gtk_text_thaw( GTK_TEXT(global->Message) );

	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( global->Message )->vadj )->upper );
}


/* insert test in Chat Box */ 
void text_insert_chat(gchar* text, guint32 lenght)  {

	guint32 i, len;

	if(global->phase < PHASE_GAMECREATE) return;

	gtk_text_freeze( GTK_TEXT(game->Chat) );

	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
	gtk_text_set_point( GTK_TEXT(game->Chat), len);
	if(len > 0)  gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, "\n", 1);
	gtk_text_insert(GTK_TEXT(game->Chat), NULL, NULL, NULL, text, lenght);
	game->chat_nb_lines++;

	while(game->chat_nb_lines > config->chat_max_lines) {

		/* if unlimited */
		if(config->chat_max_lines <= 0)  break;

		len = gtk_text_get_length( GTK_TEXT(game->Chat) );

		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(game->Chat), i) != '\n' ; i++)  {

			if(i >= len)  { /* argl... no CR founded */

				gtk_text_set_point( GTK_TEXT(game->Chat), 0);
				gtk_text_forward_delete( GTK_TEXT(game->Chat), len);
				game->chat_nb_lines = 0;
				break;
			}
		}

		if(game->chat_nb_lines)  {

			i++;
			gtk_text_set_point( GTK_TEXT(game->Chat), 0);
			gtk_text_forward_delete( GTK_TEXT(game->Chat), i);
			game->chat_nb_lines--;
		}
	}

	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
	gtk_text_set_point( GTK_TEXT(game->Chat), len);

	gtk_text_thaw( GTK_TEXT(game->Chat) );

	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj )->upper );
}


/* alloc all memory for a game */
void game_alloc()  {

	game = g_malloc0(sizeof (_game) );
}


/* free all memory for a game */
void game_free()  {

	g_free( game );
}


/* return a valid game id */
gboolean game_get_valid_player_slot(guint8 *playerslot)  {

	guint8 i;

	for(i = 0; i < MAX_PLAYERS; i++)
		if(!game->player[i].playerid) {

			*playerslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return the player slot of playerid */
gint8 get_player_slot_with_playerid(guint32 playerid)  {

	guint32 i;

	for(i = 0 ; i < MAX_PLAYERS ; i++)  {

		if(game->player[i].playerid == playerid) {

			return(i);
			break;
		}
	}

	return(-1);
}


gint8 get_playerlistcard_id_with_estate(guint32 estate)  {

	guint32 i;

	for(i = 0 ; i < data->number_playerlist_card ; i++)  {

		if(data->playerlist_card[i].estateid == estate) {

			return(i);
			break;
		}
	}

	return(-1);
}


/* return a valid command slotid */
gboolean game_get_valid_command_slot(guint8 *commandslot)  {

	guint8 i;

	for(i = 0; i < MAX_COMMANDS ; i++)
		if(!game->command[i].open) {

			*commandslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return commandslot if button command is displayed 
   return <0 if button not is not displayed           */
gint8 get_command_button_slot_with_command(guchar *command) {

	guint32 i;

	for(i = 0 ; i < MAX_COMMANDS ; i ++)  {

		if(game->command[i].open)  {

			if(! strncmp(command, game->command[i].command, strlen(command)) )
				return i;
		}
	}

	return -1;
}


/* send specific chat message, like version */
void parse_chat_specific_message(guchar *message)   {

	guchar *msg, *sendstr;
	guint32 pslot;

	pslot = get_player_slot_with_playerid( game->my_playerid );

	if(message[0] != '!') return;

	msg = message;
	msg++;

	if(! strncmp(msg, "version", 7))  {

		msg += 7;
		while( strlen(msg) )  {

			if(msg[0] != ' ') break;
			else  msg++;
		}

		if(strlen(msg) == 0  ||  !strncmp(msg, game->player[pslot].name, strlen(game->player[pslot].name) ) )  {

			sendstr = g_strdup_printf("GtkAtlantic %s\n", VERSION);
			client_send(game->connectid, sendstr, strlen(sendstr));
			g_free(sendstr);
		}
	}

}


/* sort playerlist by playerid (ascending) */
void game_sort_playerlist_by_playerid()  {

	guint32 i, j;
	_player playertmp;

	for(i = 0 ; i < MAX_PLAYERS ; i++)  {

		if(! game->player[i].playerid) continue;

		for(j = i+1 ; j < MAX_PLAYERS ; j++)  {

			if(! game->player[j].playerid) continue;

			if(game->player[j].playerid < game->player[i].playerid)  {

				memcpy(&playertmp, &game->player[i], sizeof(_player) );
				memcpy(&game->player[i], &game->player[j], sizeof(_player) );
				memcpy(&game->player[j], &playertmp, sizeof(_player) );
			}
		}
	}
}


/* write cookie on disk */
void game_write_cookie(guchar *cookie)  {

	guchar *filename, *errormsg;
	FILE *file;

	filename = g_strconcat(global->path_home, "cookie", NULL);

	file = fopen(filename, "wt");
	if(!file)  {

		errormsg = g_strdup_printf("[ERROR] Can't write cookie file at \"%s\"", filename);
		text_insert_message(errormsg, strlen(errormsg));
		g_free(errormsg);
		g_free(filename);
		return;
	}

	fprintf(file, "%s\n", connection[ game->connectid ]->host );
	fprintf(file, "%d\n", connection[ game->connectid ]->port );
	fprintf(file, "%s\n", cookie);

	g_free(filename);
	fclose(file);
}


/* return a valid card slotid */
gboolean game_get_valid_card_slot(guint8 *cardslot)  {

	guint8 i;

	for(i = 0; i < MAX_CARDS ; i++)
		if(!game->card[i].owner) {

			*cardslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return the card slot by cardid */
gint8 get_card_slot_with_cardid(guint32 cardid)  {

	guint32 i;

	for(i = 0 ; i < MAX_CARDS ; i++)  {

		if(game->card[i].cardid == cardid) {

			return(i);
			break;
		}
	}

	return(-1);
}


/* return player identifier by name of this player */
gint32 get_playerid_by_playername(guchar *name)  {

	guint32 i;

	for(i = 0 ; i < MAX_PLAYERS ; i++)  {

		if(! strcmp(game->player[i].name, name) )  {

			return( game->player[i].playerid );
			break;
		}
	}

	return(-1);
}


/* return a valid trade slotid */
gboolean game_get_valid_trade_slot(guint32 *tradeslot)  {

	guint32 i;

	for(i = 0; i < MAX_TRADES ; i++)
		if(! game->trade[i].open) {

			*tradeslot = i;
			return TRUE;
		}

	return FALSE;
}


/* return trade slot by tradeid */
gint32 get_trade_slot_with_tradeid(guint32 tradeid)  {

	guint32 i;

	for(i = 0 ; i < MAX_TRADES ; i++)  {

		if(! game->trade[i].open) continue;

		if(game->trade[i].tradeid == tradeid) {

			return(i);
			break;
		}
	}

	return(-1);
}



/* return estate identifier by name of this estate */
gint32 get_estateid_by_estatename(guchar *name)  {

	guint32 i;

	for(i = 0 ; i < data->number_estates ; i++)  {

		if(! strcmp(game->estate[i].name, name) )  {

			return(i);
			break;
		}
	}

	return(-1);
}

