/*
 *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
 *
 *
 *  Copyright (C) 2002 Rochet Sylvain
 *
 *  gtkatlantic is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <gtk/gtk.h>
#include <libxml/parser.h>

#include "engine.h"

#include "xmlparse.h"
#include "game.h"
#include "client.h"
#include "interface.h"
#include "global.h"
#include "load.h"
#include "callback.h"
#include "display.h"
#include "trade.h"

/*
ok	server
ok	client
ok	msg
ok	display
dep	updateplayerlist
ok	updategamelist
ok	playerupdate
no-need	deleteplayer
ok	estateupdate
ok	cardupdate
ok	estategroupupdate
ok	tradeupdate
ok	auctionupdate
ok	gameupdate
dep	commandlist
ok	configupdate
*/

/*
ok	.gn
ok	.gl
ok	.gj
no-need	.gd
ok	.gp
ok	.gs
	.gx
ok	.Tn
ok	.Tc
ok	.Te
ok	.Tm
ok	.Ta
ok	.Tr
ok	.r
ok	.E
ok	.t
ok	.D
no-need	.p
ok	.eb
ok	.ea
ok	.hb
ok	.hs
ok	.em
ok	.ab
ok	.jc
ok	.jp
ok	.jr
ok	.T$
ok	.T%
ok	.f
ok	.n
	.d
ok	.R
no-need	.pi
*/


void xmlparse_getgamelist_plugger(guint32 connectid) {

	xmlDocPtr doc;
	xmlNodePtr cur;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	g_free(connection[connectid]->buf_received);
	connection[connectid]->lenght_received = 0;

	if(doc == NULL)  return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, SERVER_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return;
	}

	for( cur = cur->xmlChildrenNode ; cur != NULL ; cur = cur -> next)  {

		if(! xmlStrcmp(cur->name, "server") )  xmlparse_server(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "updategamelist") )  xmlparse_updategamelist(connectid, doc, cur);
	}

	xmlFreeDoc(doc);
}




void xmlparse_game_plugger(guint32 connectid) {

	xmlDocPtr doc;
	xmlNodePtr cur;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	g_free(connection[connectid]->buf_received);
	connection[connectid]->lenght_received = 0;

	if(doc == NULL)   return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {
		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, SERVER_XMLROOTELEMENT) )  {
		xmlFreeDoc(doc);
		return;
	}

	for(cur = cur->xmlChildrenNode ; cur != NULL ; cur = cur -> next)  {

		if(! xmlStrcmp(cur->name, "gameupdate") )  xmlparse_gameupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "server") )  xmlparse_server(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "client") )  xmlparse_client(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "msg") )  xmlparse_message(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "estateupdate") )  xmlparse_estateupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "playerupdate") )  xmlparse_playerupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "auctionupdate") )  xmlparse_auctionupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "display") )  xmlparse_display(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "configupdate") )  xmlparse_configupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "tradeupdate") )  xmlparse_tradeupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "cardupdate") )  xmlparse_cardupdate(connectid, doc, cur);
		if(! xmlStrcmp(cur->name, "estategroupupdate") )  xmlparse_estategroupupdate(connectid, doc, cur);
	}

	xmlFreeDoc(doc);
}




void xmlparse_server(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guchar *tmp;

	tmp = xmlGetProp(cur, "version");
	sscanf(tmp, SERVER_VERSIONSTRING, &connection[connectid]->serverversion_major, &connection[connectid]->serverversion_minor, &connection[connectid]->serverversion_revision);
	g_free(tmp);
}




void xmlparse_client(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guchar *tmp;

	tmp = xmlGetProp(cur, "playerid");
	if( tmp )  {
		game->my_playerid = atoi( tmp );
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "cookie");
	if( tmp )  {
		game_write_cookie( tmp );
		g_free(tmp);
	}
}




void xmlparse_gameupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur) {

	guchar *sendstr, *tmp;

	tmp = xmlGetProp(cur, "status");

	if(! tmp) return;

	if(! xmlStrcmp(tmp, "config") ) {

		if(game->status != GAME_STATUS_CONFIG)  {

			game->status = GAME_STATUS_CONFIG;

			sendstr = g_strdup_printf(".n%s\n", config->nickname);
			client_send(connectid, sendstr, strlen(sendstr));
			g_free(sendstr);

			interface_create_gameconfigpage();
		}
	}

	if(! xmlStrcmp(tmp, "init") )  {

		if(game->status != GAME_STATUS_INIT)  {

			game->status = GAME_STATUS_INIT;
			interface_create_gameinitpage();
			if(! game_load_pngs() ) exit(-1);
		}
	}

	if(! xmlStrcmp(tmp, "run") )  {

		if(game->status != GAME_STATUS_RUN)  {

			game->status = GAME_STATUS_RUN;
			interface_create_gameboardpage();
		}
	}

	if(! xmlStrcmp(tmp, "end") )  {

		if(game->status != GAME_STATUS_END)  {

			game->status = GAME_STATUS_END;
		}
	}

	g_free(tmp);
}




void xmlparse_message(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *type, *text, *author, *value;

	type = xmlGetProp(cur, "type");

	if(! g_strncasecmp(type, "chat", 4) )  {

		author = xmlGetProp(cur, "author");
		value = xmlGetProp(cur, "value");
		text = g_strdup_printf("<%s> %s", author, value);
		text_insert_chat(text, strlen(text));
		g_free(text);
		g_free(author);
		if(value[0] == '!')  parse_chat_specific_message(value);
		g_free(value);
	}
	else {

		g_strup( type );
		value = xmlGetProp(cur, "value");
		text = g_strdup_printf("%s: %s", type, value);
		text_insert_message(text, strlen(text));
		g_free(text);
		g_free(value);
	}

	g_free(type);
}




void xmlparse_estateupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guint32 color[3], i;
	gchar *key, *tmp;
	guint32 id, frame, pic, stair;
	gint32 t, u;

	tmp = xmlGetProp(cur, "estateid");
	if(!tmp)  return;
	id = atoi( tmp );
	g_free(tmp);
	if(id < 0) return;

	tmp = xmlGetProp(cur, "name");
	if(tmp)  game->estate[id].name = tmp;

	tmp = xmlGetProp(cur, "color");
	if(tmp) {

		sscanf(tmp, "#%2X%2X%2X", &color[0], &color[1], &color[2]);
		game->estate[id].color[0] = color[0];
		game->estate[id].color[1] = color[1];
		game->estate[id].color[2] = color[2];
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "bgcolor");
	if(tmp) {

		sscanf(tmp, "#%2X%2X%2X", &color[0], &color[1], &color[2]);
		game->estate[id].bgcolor[0] = color[0];
		game->estate[id].bgcolor[1] = color[1];
		game->estate[id].bgcolor[2] = color[2];
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "owner");
	if(tmp)  {

		t = game->estate[id].owner = atoi(tmp);
		g_free(tmp);

		/* reset unowned cards of this estate to all players */
		for(i = 0 ; i < MAX_PLAYERS ; i++)  {

			if(! game->player[i].playerid) continue;

			frame = game->player[i].playerlist_cards_frame;
			pic = game->player[i].playerlist_cards_pic[ get_playerlistcard_id_with_estate(id) ];

			pic_set_alpha(frame, pic, data->playerlist_cards_alphaunowned);
		}

		/* set owned this estate card to player own estate */
		if(t > 0)  {

			frame = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_frame;
			pic = game->player[ get_player_slot_with_playerid(t) ].playerlist_cards_pic[ get_playerlistcard_id_with_estate(id) ];

			pic_set_alpha(frame, pic, data->playerlist_cards_alphaowned);
		}

		/* update estatelist in trade panel */
		for(i = 0 ; i < MAX_TRADES ; i++)  {

			if(! game->trade[i].open)  continue;
			if(game->trade[i].current_component != TRADE_CURRENT_COMPONENT_ESTATE)  continue;

			trade_rebuild_subcomponent(i);
		}
	}

	tmp = xmlGetProp(cur, "houses");
	if(tmp)  {

		t = game->estate[id].houses = atoi(tmp);
 		g_free(tmp);

		frame = game->board_frame;

		if(t <= 0)   {

			/* houses */
			pic = game->estate[id].house_pic;
			pic_unshow(frame, pic);
		}
		else  if(t > 0  &&  data->estate[id].type_house == TYPE_HOUSE_HORIZONTAL)  {

			/* houses */
			pic = game->estate[id].house_alpha;
			pic_set_width(frame, pic, data->pngfile_horiz_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_horiz_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_horiz_house_alpha[t]);

			pic = game->estate[id].house_pic;
			pic_set_width(frame, pic, data->pngfile_horiz_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_horiz_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_horiz_house_image[t]);
			pic_show(frame, pic);
		}
		else  if(t > 0  &&  data->estate[id].type_house == TYPE_HOUSE_VERTICAL)  {

			/* houses */
			pic = game->estate[id].house_alpha;
			pic_set_width(frame, pic, data->pngfile_vert_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_vert_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_vert_house_alpha[t]);

			pic = game->estate[id].house_pic;
			pic_set_width(frame, pic, data->pngfile_vert_house_width[t]);
			pic_set_height(frame, pic, data->pngfile_vert_house_height[t]);
			pic_set_buff(frame, pic, data->pngfile_vert_house_image[t]);
			pic_show(frame, pic);
		}
	}

	tmp = xmlGetProp(cur, "houseprice");
	if(tmp)  {
		game->estate[id].houseprice = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "sellhouseprice");
	if(tmp)  {
		game->estate[id].sellhouseprice = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "mortgageprice"); 
	if(tmp)  {
		game->estate[id].mortgageprice = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "unmortgageprice");
	if(tmp)  {
		game->estate[id].unmortgageprice = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "mortgaged");
	if(tmp)  {

		u = game->estate[id].mortgaged = atoi(tmp);
		g_free(tmp);

		t = game->estate[id].owner;
		frame = game->board_frame;

		if(t > 0  &&  !u)  {

			/* stair */
			stair = game->player[ get_player_slot_with_playerid(t) ].buffer_stair;

			pic = game->estate[id].stair_alpha;
			pic_set_width(frame, pic, data->pngfile_stair_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_alpha[stair]);

			pic = game->estate[id].stair_pic;
			pic_set_width(frame, pic, data->pngfile_stair_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_image[stair]);
			pic_show(frame, pic);
		}
		else  if(t > 0  &&  u)  {

			/* stair */
			stair = game->player[ get_player_slot_with_playerid(t) ].buffer_stair;

			pic = game->estate[id].stair_alpha;
			pic_set_width(frame, pic, data->pngfile_stair_m_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_m_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_m_alpha[stair]);

			pic = game->estate[id].stair_pic;
			pic_set_width(frame, pic, data->pngfile_stair_m_width[stair]);
			pic_set_height(frame, pic, data->pngfile_stair_m_height[stair]);
			pic_buff(frame, pic, data->pngfile_stair_m_image[stair]);
			pic_show(frame, pic);
		}
		else /* if( t <= 0 ) */ {

			/* stair */
			pic_unshow(frame, game->estate[id].stair_pic);
		}
	}

	tmp = xmlGetProp(cur, "group");
	if(tmp)  {
		game->estate[id].group = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "can_be_owned");
	if(tmp)  {
		game->estate[id].can_be_owned = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "can_toggle_mortgage");
	if(tmp)  {
		game->estate[id].can_toggle_mortgage = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "can_buy_houses");
	if(tmp)  {
		game->estate[id].can_buy_houses = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "can_sell_houses");
	if(tmp)  {
		game->estate[id].can_sell_houses = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "money");
	if(tmp)  {
		game->estate[id].money = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "price");
	if(tmp)  {
		game->estate[id].price = atoi(tmp);
		g_free(tmp);
	}

	for(i = 0 ; i <= 5 ; i++) {
   
		key = g_strdup_printf("rent%d", i);
		tmp = xmlGetProp(cur, key);

		if(tmp)  {
			game->estate[id].rent[i] = atoi(tmp);
			g_free(tmp);
		}
		g_free(key);
	}
}




void xmlparse_playerupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guint32 playerid, i;
	gint8 id;
	gchar *sendstr, *txt[10];
	gint32 row;
	guchar *tmp;
	GtkWidget *Combo;
	GList *list;


	tmp = xmlGetProp(cur, "playerid");
	if(!tmp) return;
	playerid = atoi(tmp);
	g_free(tmp);
	if(!playerid) return;

	id = get_player_slot_with_playerid(playerid);

	/* create new player */
	if(id < 0)  {

		if(! game_get_valid_player_slot(&id) ) return;

		memset(&game->player[id], 0, sizeof(_player) );
		game->player[id].playerid = playerid;
		game->nb_players++;
	}

	tmp = xmlGetProp(cur, "game");
	if(tmp)  {

		game->player[id].game = atoi(tmp);
		g_free(tmp);

		/* delete a player */
		if(game->player[id].game < 0)  {

			if(game->player[id].name) g_free(game->player[id].name);
			if(game->player[id].host) g_free(game->player[id].host);
			if(game->player[id].image) g_free(game->player[id].image);
			memset(&game->player[id], 0, sizeof(_player) );
			game->nb_players--;
		}
	}

	tmp = xmlGetProp(cur, "host");
	if(tmp)  {

		if(game->player[id].host) g_free(game->player[id].host);
		game->player[id].host = tmp;
	}

	tmp = xmlGetProp(cur, "master");
	if(tmp)  {

		game->player[id].master = atoi(tmp);
		g_free(tmp);

		if(game->player[id].master  &&  game->my_playerid == playerid)
			game->i_am_the_master = TRUE;
		if(!game->player[id].master  &&  game->my_playerid == playerid)
			game->i_am_the_master = FALSE;
	}

	tmp = xmlGetProp(cur, "image");
	if(tmp)  {

		if(game->player[id].image) g_free(game->player[id].image);
		game->player[id].image = tmp;
	}

	tmp = xmlGetProp(cur, "name");
	if(tmp)  {

		if(game->player[id].name) g_free(game->player[id].name);
		game->player[id].name = g_strndup(tmp, MAX_LENGTH_NICK);
		g_free(tmp);

		/* playerlist name */
		if(game->player[id].playerlist_LabelNamePlayer)
			gtk_label_set_text(GTK_LABEL(game->player[id].playerlist_LabelNamePlayer), game->player[id].name);

		/* popdown string trade combo */
		Combo = gtk_object_get_data(GTK_OBJECT(global->MainVerticalBox), "trade_combo_listplayer");
		if(Combo)  {

			list = NULL;
			for(i = 0 ; i < MAX_PLAYERS ; i++)  {

				if(! game->player[i].playerid) continue;
				if(game->player[i].playerid  ==  game->my_playerid) continue;
	
				list = g_list_append(list, game->player[i].name);
			}
			gtk_combo_set_popdown_strings(GTK_COMBO(Combo), list);
			g_list_free(list);
		}

		/* playerlist in trade panel */
		for(i = 0 ; i < MAX_TRADES ; i++)  {

			if(! game->trade[i].open)  continue;

			trade_rebuild_playerlist(i);
		}

		/* update sub component players in trade panel */
		for(i = 0 ; i < MAX_TRADES ; i++)  {

			if(! game->trade[i].open)  continue;

			trade_rebuild_subcomponent(i);
		}
	}

	tmp = xmlGetProp(cur, "money");
	if(tmp)  {

		game->player[id].money = atoi(tmp);

		if(game->player[id].playerlist_LabelMoneyPlayer)
			gtk_label_set_text(GTK_LABEL(game->player[id].playerlist_LabelMoneyPlayer), tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "location");
	if(tmp)  {

		game->player[id].location = atoi(tmp);
		g_free(tmp);

		pic_set_x(game->board_frame, game->player[id].token_pic, data->estate[ game->player[id].location ].xtoken);
		pic_set_y(game->board_frame, game->player[id].token_pic, data->estate[ game->player[id].location ].ytoken);

		sendstr = g_strdup_printf(".t%d\n", game->player[id].location);
		client_send(connectid, sendstr, strlen(sendstr));
		g_free(sendstr);
	}

	tmp = xmlGetProp(cur, "jailed");
	if(tmp)  {
		game->player[id].jailed = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "directmove");
	if(tmp)  {
		game->player[id].directmove = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "bankrupt");
	if(tmp)  {
		game->player[id].bankrupt = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "hasturn");
	if(tmp)  {
		game->player[id].hasturn = atoi(tmp);
		g_free(tmp);

		if(game->player[id].playerlist_LabelNamePlayer)  {

			if(game->player[id].hasturn)  {

				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementNamePlayer), "namelabel") ,  global->style_player_turn);
				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementMoneyPlayer), "moneylabel") ,  global->style_player_turn);
			}
			else  {

				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementNamePlayer), "namelabel") ,  global->style_player_normal);
				gtk_widget_set_style( gtk_object_get_data(GTK_OBJECT(game->player[id].playerlist_AlignementMoneyPlayer), "moneylabel") ,  global->style_player_normal);
			}
		}
	}

	tmp = xmlGetProp(cur, "can_roll");
	if(tmp)  {
		game->player[id].can_roll = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "can_buyestate");
	if(tmp)  {
		game->player[id].can_buyestate = atoi(tmp);
		g_free(tmp);
	}

	game_sort_playerlist_by_playerid();

	/* build player list */
	if(global->phase == PHASE_GAMECREATE)  {

		gtk_clist_freeze(GTK_CLIST(game->PlayerCList));
		gtk_clist_clear(GTK_CLIST(game->PlayerCList));

		for(i = 0 ; i < MAX_PLAYERS ; i++)  {

			if(game->player[i].playerid)  {

				txt[0] = game->player[i].name;
				txt[1] = game->player[i].host;
				row = gtk_clist_append(GTK_CLIST(game->PlayerCList), txt);
				gtk_clist_set_selectable(GTK_CLIST(game->PlayerCList), row ,FALSE);			
			}
		}

		gtk_clist_thaw(GTK_CLIST(game->PlayerCList));
	}
}




void xmlparse_auctionupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	GtkWidget *text, *entry;
	guint32 actor = 0, auctionid, estateid, highbid = 0, highbidder, len, status;
	guchar *ptr, *message, *tmp;

	tmp = xmlGetProp(cur, "auctionid");
	if(!tmp) return;
	auctionid = atoi(tmp);
	g_free(tmp);

	/* initialise auction */
	tmp = xmlGetProp(cur, "actor");
	if(tmp)  {
		actor = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "estateid");
	if(tmp)  {

		estateid = atoi(tmp);
		g_free(tmp);

		display_clear_all();
		interface_create_auctionbox();

		text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");
		entry = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_entry");

		ptr = g_strdup_printf("%d", auctionid);
		gtk_object_set_data_full(GTK_OBJECT(entry), "auctionid", ptr, g_free);

		message = g_strdup_printf("%s auction estate %s", game->player[ get_player_slot_with_playerid( actor ) ].name, game->estate[ estateid ].name);
		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
		g_free(message);
	}

	/* bid */
	tmp = xmlGetProp(cur, "highbid");
	if(tmp)  {
		highbid = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "highbidder");
	if(tmp)  {

		highbidder = atoi(tmp);
		g_free(tmp);

		message = g_strdup_printf("%s bid %d", game->player[ get_player_slot_with_playerid(highbidder) ].name, highbid);

		text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

		len = gtk_text_get_length( GTK_TEXT(text) );
		gtk_text_set_point( GTK_TEXT(text), len);
		if(len > 0)  gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, "\n", 1);
		gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );

		g_free(message);
	}

	/* status */
	tmp =  xmlGetProp(cur, "status");
	if(tmp)  {

		status = atoi(tmp);
		g_free(tmp);

		switch( status )  {

			case 1:
				text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

				message = g_strdup("\n<- going once ->");
				gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
				g_free(message);
				break;

			case 2:
				text = gtk_object_get_data(GTK_OBJECT(game->BoardCenter), "auction_text");

				message = g_strdup("\n<- going twice ->");
				gtk_text_insert(GTK_TEXT(text), NULL, NULL, NULL, message, strlen(message) );
				g_free(message);
				break;

			case 3:
				display_clear_all();
				break;
		}
	}
}




void xmlparse_display(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	xmlNodePtr cur2;
	gint32 estateid;
	guchar *tmp, *caption, *command;
	gboolean enable;

	if( game->status != GAME_STATUS_RUN)  return;

	display_init();

	tmp = xmlGetProp(cur, "cleartext");
	if(tmp)  {
		if( atoi(tmp) )  display_clear_text();
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "clearbuttons");
	if(tmp)  {
		if( atoi(tmp) )  display_clear_buttons();
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "estateid");
	if(tmp)   {

		estateid = atoi(tmp);
		g_free(tmp);

		if(estateid < 0)  display_clear_title();
		else  display_estate(estateid);
	}

	tmp = xmlGetProp(cur, "text");
	if(tmp)  {

		if( strlen(tmp) > 0)  display_text(tmp);
		g_free(tmp);
	}

	/* buttons */
	for(cur2 = cur->xmlChildrenNode ; cur2 != NULL ; cur2 = cur2 -> next)  {

		if(! xmlStrcmp(cur2->name, "button") )  {

			caption = xmlGetProp(cur2, "caption");
			command = xmlGetProp(cur2, "command");

			tmp = xmlGetProp(cur2, "enabled");		
			enable = atoi(tmp);
			g_free(tmp);

			display_button(caption, command, enable);
			g_free(caption);
			g_free(command);
		}
	}
}




void xmlparse_configupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	GtkWidget *CButton;
	xmlNodePtr cur2;
	guchar *text, *tmp, *title, *command;

	for( cur2 = cur->xmlChildrenNode ; cur2 != NULL ; cur2 = cur2 -> next)  {

		if(! xmlStrcmp(cur2->name, "option") )  {

			tmp = xmlGetProp(cur2, "type");
			if(!tmp)  continue;
			if(xmlStrcmp(tmp, "bool") )  {

				g_free(tmp);
				continue;
			}
			g_free(tmp);

			command = xmlGetProp(cur2, "command");
			if(!command)  continue;

			/* create it if not created */
			if(! gtk_object_get_data(GTK_OBJECT(game->GameConfigBox), command) )  {

				title = xmlGetProp(cur2, "title");
				if(!title)  continue;

				CButton = gtk_check_button_new_with_label(title);
				gtk_signal_connect(GTK_OBJECT(CButton), "toggled", (GtkSignalFunc)Callback_toggle_boolean_gameoption, NULL);
				gtk_box_pack_start(GTK_BOX(game->GameConfigBox), CButton, FALSE, FALSE, 0);

				/* store command under CButton */
				text = g_strdup(command);
				gtk_object_set_data_full(GTK_OBJECT(CButton), "command", text, g_free);

				/* save the pointer of button to use in the future */
				gtk_object_set_data(GTK_OBJECT(game->GameConfigBox), command, CButton);

				g_free(title);
			}

			/* modify */
			CButton = gtk_object_get_data(GTK_OBJECT(game->GameConfigBox), command);

			tmp = xmlGetProp(cur2, "value");
			if(tmp)   {

				if( atoi(tmp) ) 
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CButton), TRUE);
				else
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(CButton), FALSE);

				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "edit");
			if(tmp)   {

				if( atoi(tmp) )
					gtk_widget_set_sensitive(CButton, TRUE);
				else
					gtk_widget_set_sensitive(CButton, FALSE);

				g_free(tmp);
			}

			g_free(command);
		}
	}

	gtk_widget_show_all(game->GameConfigBox);
}




void xmlparse_tradeupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	xmlNodePtr cur2;
	guchar *tmp;
	guint32 tradeid = 0, playerid = 0, actor = 0, cardid = 0, estateid = 0;
	guint32 targetplayer = 0, playerfrom = 0, playerto = 0, money = 0;
	gboolean accept = FALSE;

	if( game->status != GAME_STATUS_RUN)  return;

	tmp = xmlGetProp(cur, "tradeid");
	if(tmp)  {
		tradeid = atoi(tmp);
		g_free(tmp);
	}
	else  return;

	tmp = xmlGetProp(cur, "actor");
	if(tmp)   {
		actor = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "type");
	if(tmp)  {

		if(! xmlStrcmp(tmp, "new") )
			trade_initnew(tradeid);

		/* create trade panel if wasn't created (server bug I suppose) */
		if(get_trade_slot_with_tradeid(tradeid) < 0)
			trade_initnew(tradeid);

		if(! xmlStrcmp(tmp, "rejected") )
			trade_destroy(tradeid);

		if(! xmlStrcmp(tmp, "completed") )
			trade_destroy(tradeid);

		g_free(tmp);
	}


	for(cur2 = cur->xmlChildrenNode ; cur2 != NULL ; cur2 = cur2 -> next)  {

		/* -- player -- */
		if(! xmlStrcmp(cur2->name, "tradeplayer") )  {

			tmp = xmlGetProp(cur2, "playerid");
			if(tmp)   {
				playerid = atoi(tmp);
				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "accept");
			if(tmp)   {
				accept = atoi(tmp);
				g_free(tmp);
			}

			trade_update_player(tradeid, playerid, accept);
		}

		/* -- card -- */
		if(! xmlStrcmp(cur2->name, "tradecard") )  {

			tmp = xmlGetProp(cur2, "cardid");
			if(tmp)   {
				cardid = atoi(tmp);
				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "targetplayer");
			if(tmp)   {
				targetplayer = atoi(tmp);
				g_free(tmp);
			}

			trade_update_card(tradeid, cardid, targetplayer);
		}

		/* -- estate -- */
		if(! xmlStrcmp(cur2->name, "tradeestate") )  {

			tmp = xmlGetProp(cur2, "estateid");
			if(tmp)   {
				estateid = atoi(tmp);
				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "targetplayer");
			if(tmp)   {
				targetplayer = atoi(tmp);
				g_free(tmp);
			}

			trade_update_estate(tradeid, estateid, targetplayer);
		}

		/* -- money -- */
		if(! xmlStrcmp(cur2->name, "trademoney") )  {

			tmp = xmlGetProp(cur2, "playerfrom");
			if(tmp)   {
				playerfrom = atoi(tmp);
				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "playerto");
			if(tmp)   {
				playerto = atoi(tmp);
				g_free(tmp);
			}

			tmp = xmlGetProp(cur2, "money");
			if(tmp)   {
				money = atoi(tmp);
				g_free(tmp);
			}

			trade_update_money(tradeid, playerfrom, playerto, money);
		}
	}

}




void xmlparse_cardupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guchar *tmp;
	gint32 cardid = 0, owner = 0, i;
	gint8 cardslot = 0;

	if( game->status != GAME_STATUS_RUN)  return;

	tmp = xmlGetProp(cur, "cardid");
	if(tmp)  {
		cardid = atoi(tmp);
		g_free(tmp);
	}

	tmp = xmlGetProp(cur, "owner");
	if(tmp)  {
		owner = atoi(tmp);
		g_free(tmp);
	}

	/* create new card slot */
	if(owner >= 0)  {

		cardslot = get_card_slot_with_cardid(cardid);
		if(cardslot < 0)  if(! game_get_valid_card_slot(&cardslot) )  return;

		game->card[cardslot].cardid = cardid;
		game->card[cardslot].owner = owner;
	}

	/* destroy a card slot */
	else  {

		cardslot = get_card_slot_with_cardid(cardid);
		if(cardslot < 0)  return;

		game->card[cardslot].cardid = 0;
		game->card[cardslot].owner = 0;
	}

	/* update component list in trade panel */
	for(i = 0 ; i < MAX_TRADES ; i++)  {

		if(! game->trade[i].open)  continue;

		trade_rebuild_component(i);
		trade_rebuild_subcomponent(i);
	}
}




void xmlparse_estategroupupdate(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	guchar *tmp;
	gint32 groupid = -1;

	tmp = xmlGetProp(cur, "groupid");
	if(tmp)  {
		groupid = atoi(tmp);
		g_free(tmp);
	}

	if(groupid < 0)  return;

	if(game->group[groupid].name)  g_free(game->group[groupid].name);

	game->group[groupid].name = xmlGetProp(cur, "name");
}




void xmlparse_updategamelist(guint32 connectid, xmlDocPtr doc, xmlNodePtr cur)  {

	gchar *type, *txt[10], *host, *port, *name, **text[2], *tmp;
	gint32 id, test, i;
	xmlNodePtr cur2;

	type = xmlGetProp(cur, "type");
	if(!type) return;

	if(! xmlStrcmp(type, "full") )  {

		gtk_clist_freeze(GTK_CLIST(global->GameCList));
		gameclist_title_showgames();

		host = g_strdup( g_strstrip(connection[connectid]->host) );
		port = g_strdup_printf("%d", connection[connectid]->port);

		/* remove all game from same host & port */
		text[0] = g_malloc0(8192);
		text[1] = g_malloc0(8192);
		for(i = 0 ;   ; i++) {

			test = gtk_clist_get_text(GTK_CLIST(global->GameCList), i, 0, text[0]);
			if(!test) break;
			test = gtk_clist_get_text(GTK_CLIST(global->GameCList), i, 5, text[1]);
			if(!test) break;

			if(! strncmp(host, *text[0], strlen(host))  &&  ! strncmp(port, *text[1], strlen(port)))  {

				gtk_clist_remove(GTK_CLIST(global->GameCList), i);
				i--;
			}
		}
		g_free(text[0]);
		g_free(text[1]);


		for(cur2 = cur->xmlChildrenNode ; cur2 != NULL ; cur2 = cur2 -> next )  {

			if(! xmlStrcmp(cur2->name, "game") )  {

				tmp = xmlGetProp(cur2, "canbejoined");
				if(tmp  &&  atoi(tmp) == 0 )  {

					g_free(tmp);
					continue;
				}
				if(tmp) g_free(tmp);

				/* ONLY TEMPORARLY ! */
				tmp = xmlGetProp(cur2, "gametype");
				if(xmlStrcmp(tmp, "city")  &&  xmlStrcmp(tmp, "french_city") )  {

					g_free(tmp);
					continue;
				}
				g_free(tmp);

				tmp = xmlGetProp(cur2, "id");
				id = atoi(tmp);
				g_free(tmp);

				name = xmlGetProp(cur2, "name");

				txt[0] = g_strdup(host);
				txt[1] = g_strdup_printf("%d.%d.%d",  connection[connectid]->serverversion_major,  connection[connectid]->serverversion_minor,  connection[connectid]->serverversion_revision);
				if(id < 0)
					txt[2] = g_strdup_printf("Create a new %s game", name);
				else
					txt[2] = g_strdup_printf("Join %s game #%d", name, id);
				txt[3] = xmlGetProp(cur2, "description");
				txt[4] = xmlGetProp(cur2, "players");
				txt[5] = g_strdup(port);
				txt[6] = xmlGetProp(cur2, "gametype");
				txt[7] = xmlGetProp(cur2, "id");

				gtk_clist_append(GTK_CLIST(global->GameCList), txt);

				for(i = 0 ; i < 8 ; i++)
					g_free(txt[i]);
				g_free(name);
			}
		}

		gtk_clist_thaw(GTK_CLIST(global->GameCList));
		g_free(host);
		g_free(port);

		client_disconnect(connectid);
	}

	g_free(type);
}




void xmlparse_metaserver(guint32 connectid) {

	gchar *txt[10], *text;
	guchar *tmp, *host, *portstr;
	xmlDocPtr doc;
	xmlNodePtr cur;
	guint32 i, do_close = TRUE;

	doc = xmlParseMemory(connection[connectid]->buf_received,  connection[connectid]->lenght_received);

	g_free(connection[connectid]->buf_received);
	connection[connectid]->lenght_received = 0;

	if(doc == NULL)  return;

	cur = xmlDocGetRootElement(doc);
	if (cur == NULL) {

		xmlFreeDoc(doc);
		return;
	}

	if( xmlStrcmp(cur->name, METASERVER_XMLROOTELEMENT) )  {

		xmlFreeDoc(doc);
		return;
	}

	gtk_clist_freeze(GTK_CLIST(global->GameCList));

	if(connection[connectid]->type == CONNECT_TYPE_META_GETLIST)  {

		gameclist_title_showservers();
		gtk_clist_clear(GTK_CLIST(global->GameCList));
	}
	else if(connection[connectid]->type == CONNECT_TYPE_META_GETGAME)  {

		gameclist_title_showgames();
	}

	for(cur = cur->xmlChildrenNode ; cur != NULL ; cur = cur -> next)  {

		if(! xmlStrcmp(cur->name, "metaserver") )  {

			tmp = xmlGetProp(cur, "version");
			text = g_strdup_printf("Atlantic metaserver version \"%s\"", tmp);
			g_free(tmp);
			text_insert_message(text, strlen(text));
			g_free(text);

			do_close = FALSE;
		}

		if(! xmlStrcmp(cur->name, "msg") )  {

			tmp = xmlGetProp(cur, "value");
			text = g_strdup_printf("%s", tmp);
			g_free(tmp);
			interface_create_messagewin(text);
			g_free(text);

			do_close = FALSE;
		}

		if(! xmlStrcmp(cur->name, "server") )  {

			if(connection[connectid]->type == CONNECT_TYPE_META_GETLIST)  {

				txt[0] = xmlGetProp(cur, "host");
				txt[1] = xmlGetProp(cur, "port");
				txt[2] = xmlGetProp(cur, "version");
				txt[3] = xmlGetProp(cur, "users");
				txt[4] = g_strdup("");
				txt[5] = g_strdup("");
				txt[6] = g_strdup("");
				txt[7] = g_strdup("");

				gtk_clist_append(GTK_CLIST(global->GameCList), txt);

				for(i = 0 ; i < 8 ; i++)
					g_free(txt[i]);
			}
			else if(connection[connectid]->type == CONNECT_TYPE_META_GETGAME)  {

				host = xmlGetProp(cur, "host");
				portstr = xmlGetProp(cur, "port");
				create_connection_get_games(host, atoi(portstr) );
				g_free(host);
				g_free(portstr);
			}
		}
	}

	gtk_clist_thaw(GTK_CLIST(global->GameCList));
	xmlFreeDoc(doc);
	if(do_close)  client_disconnect(connectid);
}



